<?php
namespace SupplyTek\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\FilterCriteria;
use SupplyTek\Utilities\SessionHandler;
use SupplyTek\Utilities\CrudSupplyTek;

/**
 * Class handling expenses
 * 
 * @author Danilo D. Vaz
 */
class ExpensesController 
{
    protected $crud;
    protected $sessionHandler;

    public function __construct (CrudSupplyTek $crudSupplyTek, SessionHandler $sessionHandler) 
    {
        $this->crud = $crudSupplyTek;
        $this->sessionHandler = $sessionHandler;
    }

    /**
     * Retrives expenses by category in the month
     * 
     * Get total amount and amount spent and calculates the available amount 
     * for unit, according to the period.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function getExpensesByCategory (Request $request, Response $response)
    {
        try {
            $params['OPERATOR_ID'] = $this->sessionHandler->getSessionData('CDOPERADOR');
            $filterCriteria = $request->getFilterCriteria();
            
            $filter = $this->crud->processFilter($filterCriteria);
            $params['BRANCH'] = $this->getBranch($filter);
            list($params['FIRST_MONTH'], $params['LAST_MONTH']) = $this->getPeriod($filter);
            
            $requestAmountSpent = $this->buildNewRequest('amount-spent-category', $filterCriteria, $params);
            $categoryWithAmountSpent = $this->crud->find($requestAmountSpent);
            $params['CATEGORY_ID'] = $this->getCategoryIdList($categoryWithAmountSpent['data']);

            $expensesCategoryMonth = $this->crud->executeQuery('expenses-by-category', $params);
            
            $expensesByCategory = $this->buildExpensesByCategory($expensesCategoryMonth, $params['FIRST_MONTH']);

            $data = array(
                "data" => $expensesByCategory,
                "pagination" => $categoryWithAmountSpent['pagination']
            );
            
            $response->setMetadata($data);
        } catch (\Exception $e) {
            throw new \Exception ($e->getMessage(), 500);
        }
    }
    
    /**
     * Function that process unit 
     * 
     * Receives the filter sent by frontend and checks the unit. If not informed
     * any unit, returns the session unit.
     * 
     * @param array $filter Array with filter sent by frontend
     *
     * @return string
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function getBranch ($filter)
    {
        if (isset($filter['BRANCH'])) {
            return $filter['BRANCH'];
        }
        
        return $this->sessionHandler->getSessionData('CDFILIAL');
    }
    
    /**
     * Function that process period 
     * 
     * Receives the filter sent by frontend and checks the period. If not informed
     * any period, returns the current month. Execute other validations in period.
     * Returns the period of 12 months
     * 
     * @param array $filter Array with filter sent by frontend
     *
     * @return array
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function getPeriod ($filter)
    {
        if (isset($filter['PERIOD'])) {
            $date = $filter['PERIOD'];
        } else {
            $date = $period = date('m/y');
        }
        
        $date = \DateTime::createFromFormat('m/y', $date);
        
        if (!$date) {
            throw new \Exception("Wrong date format.");
        }

        $date->modify('first day of this month');
        $lastMonth = $date->format('d/m/Y');
        
        $date->modify('-11 month');
        $firstMonth = $date->format('d/m/Y');
        
        return array($firstMonth, $lastMonth);
    }
    
    /**
     * Function that build a new request
     * 
     * Build a new request for the datasource sent in parameter
     * 
     * @param string $dataSourceName String with datasource name
     * @param FilterCriteria $filterCriteria Object sent by frontend
     * @param array $params Array with conditions
     *
     * @return Request
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function buildNewRequest ($dataSourceName, $filterCriteria, $params)
    {
        $newFilterCriteria = new FilterCriteria(
            $dataSourceName,
            array(),
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        
        foreach ($params as $columnName=>$value) {
            $newFilterCriteria->addCondition($columnName, $value);
        }
        
        return new Request\Filter($newFilterCriteria, '', '', '');
    }
    
    /**
     * Function that get a list of categories 
     * 
     * Generates a list with category id and concatenate in the string
     * 
     * @param Array $amountSpent Array provided by database
     *
     * @return string
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function getCategoryIdList ($amountSpent)
    {
        $categoryIdList = array();
        
        foreach ($amountSpent as $category) {
            array_push($categoryIdList, $category['CDCATEGORIA']);
        }
        
        return implode('_', $categoryIdList);
    }
    
    /**
     * Function that get a template used
     * 
     * Receives the parameters and build a template used in response
     * 
     * @param string $categoryId String with category id
     * @param string $category String with category name
     * @param string $firstMonth String with first month of the period (DD/MM/YYYY)
     *
     * @return array
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function getTemplate ($categoryId, $category, $firstMonth)
    {
        $firstMonth = \DateTime::createFromFormat('d/m/Y', $firstMonth);
        
        for ($i = 0; $i < 12; $i ++) {
            $expenses[$i]['MONTH'] = $firstMonth->format('m/y');
            $expenses[$i]['VALUE'] = 0;
            $firstMonth->modify('1 month');
        }
        
        $template = array(
            "CATEGORY" => $category,
            "EXPENSES" => $expenses,
            "ID" => $categoryId
        );
        
        return $template;
    }
    
    /**
     * Function that build the data for the response
     * 
     * Receives the expenses by category from each month and build a array from 
     * the template with information of 12 months.
     * 
     * @param array $expensesCategoryMonth Array with expenses
     * @param string $firstMonth String with first month 
     *
     * @return array
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function buildExpensesByCategory ($expensesCategoryMonth, $firstMonth)
    {
        $expensesByCategory = array();
        
        if (count($expensesCategoryMonth)) {
            $categoryId = $expensesCategoryMonth[0]['CDCATEGORIA'];
            $expenses = $this->getTemplate($categoryId, $expensesCategoryMonth[0]['NMCATEGORIA'], $firstMonth);
        
            foreach ($expensesCategoryMonth as $expenseCategory) {
                if ($categoryId != $expenseCategory['CDCATEGORIA']) {
                    array_push($expensesByCategory, $expenses);
                    $expenses = $this->getTemplate($expenseCategory['CDCATEGORIA'], $expenseCategory['NMCATEGORIA'], $firstMonth);
                    $categoryId = $expenseCategory['CDCATEGORIA'];
                }
                
                foreach ($expenses['EXPENSES'] as &$month) {
                    if ($expenseCategory['MES_GASTO'] == $month['MONTH']) {
                        $month['VALUE'] = $expenseCategory['TOTAL_GASTO'];
                    }
                }
            }
            
            array_push($expensesByCategory, $expenses);
        }
        
        return $expensesByCategory;
    }
    
    /**
     * Retrives total value by category in the month
     * 
     * Get total the total consumption value by category in 
     * the month of all units according to the operator logged in
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Grazielle Ferreira Antunes
     */
    public function getExpensesByCategoryOperation (Request $request, Response $response)
    {
        try {
            $params['OPERATOR_ID'] = $this->sessionHandler->getSessionData('CDOPERADOR');
            $filterCriteria = $request->getFilterCriteria();
            
            $filter = $this->crud->processFilter($filterCriteria);
            list($firsMonth, $lastMonth) = $this->getPeriod($filter);
            $params['PERIOD'] = $lastMonth;

            $expensesByCategoryOperation = $this->crud->find($request, $params);

            $response->setMetadata($expensesByCategoryOperation);
        } catch (\Exception $e) {
            throw new \Exception ($e->getMessage(), 500);
        }
    }
    
}