<?php
namespace SupplyTek\Services;

/**
 * Notification Model
 * 
 * Connects to the Database to handle queries 
 * related to notifications processes. 
 *  
 * @author Bruna Chaves Carneiro
 */
class NotificationsService
{
    protected $connection;
    protected $newCodeSC;

    /**
     * Class constructor
     * 
     * Initialize an object's properties upon creation of the object.
     * 
     * @param object $entityManager Object to manage database connection and queries
     * @param object $newCodeSC Object to generate new code without commiting
     * 
     * @return void
     * 
     * @access public
     * @author Bruna Chaves Carneiro
     */
    public function __construct($entityManager, $newCodeSC)
    {
        $this->connection = $entityManager->getConnection();
        $this->newCodeSC = $newCodeSC;
    }

    /**
     * Function that begin database transaction
     * 
     * @return void
     * 
     * @access public
     * @author Bruna Chaves Carneiro
     */
    public function beginTransaction () {
        $this->connection->beginTransaction();
    }
    
    /**
     * Function that commit database modifications and close transaction
     * 
     * @return void
     * 
     * @access public
     * @author Bruna Chaves Carneiro
     */
    public function commit () {
        $this->connection->commit();
    }
    
    /**
     * Function that rollback database modifications and close transaction
     * 
     * @return void
     * 
     * @access public
     * @author Bruna Chaves Carneiro
     */
    public function rollback () {
        $this->connection->rollback();
    }
    
    /**
     * Insert new notifications on the system
     * 
     * Receive parameters from the controller and insert a new notification.
     *
     * @param array $parameters Array with needed data to insert notification
     *
     * @return array
     *
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function insertNotification($parameters) 
    {
        $newCode = $this->newCodeSC->getCodeByTableName('NOTIFICSUPPLY', $parameters['ORG_ID']);
        
        $sql = "INSERT INTO NOTIFICSUPPLY 
                (
                    NRNOTIFICSUPPLY, 
                    CDFILIAL, 
                    CDOPERADOR, 
                    NMNOTIFICACAO, 
                    DSTEXTNOTIFIC, 
                    IDNOTIFICLIDO, 
                    NRORG, 
                    IDATIVO, 
                    DTINCLUSAO, 
                    CDOPERINCLUSAO
                ) 
                VALUES
                (
                    :NOTIFICATION_SUPPLY_ID, 
                    :UNIT_ID, 
                    :OPERATOR_ID, 
                    :NOTIFICATION_NAME, 
                    :NOTIFICATION_TEXT, 
                    'N', 
                    :ORG_ID, 
                    'S', 
                    SYSDATE, 
                    :LOGGED_OPERATOR_ID
                )";
                
        $params = array(
            "NOTIFICATION_SUPPLY_ID" => (string)$newCode,
            "UNIT_ID" => $parameters["UNIT_ID"],
            "OPERATOR_ID" => $parameters["OPERATOR_ID"],
            "NOTIFICATION_NAME" => $parameters["NOTIFICATION_NAME"],
            "NOTIFICATION_TEXT" => $parameters["NOTIFICATION_TEXT"],
            "ORG_ID" => $parameters["ORG_ID"],
            "LOGGED_OPERATOR_ID" => $parameters["LOGGED_OPERATOR_ID"]
        );

        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
    }

    /**
     * Updates the notification as read
     * 
     * Receive parameters from the controller and update the notification to read.
     *
     * @param array $parameters Array with inputted login data.
     *
     * @return array
     *
     * @access public
     * @author Bruna Chaves Carneiro
     */
    public function updateNotifications($parameters)
    {
        $sql = "UPDATE 
                    NOTIFICSUPPLY 
                SET 
                    IDNOTIFICLIDO = :READ 
                WHERE 
                    ( NRNOTIFICSUPPLY = :ID OR :ID = 'T' )
                    AND CDFILIAL = :UNIT_ID
                    AND CDOPERADOR = :OPERATOR_ID";
        
        $params = array(
            "READ" => $parameters["READ"],
            "ID" => $parameters["ID"],
            "UNIT_ID" => $parameters["UNIT_ID"],
            "OPERATOR_ID" => $parameters["OPERATOR_ID"]
        );

        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
    }
    
    /**
     * Delete notifications
     * 
     * Receives parameters from the controller and deletes selected notifications.
     * 
     * @param string $notificationId String with notification id.
     * @param string $operatorId String with operator id.
     * @param string $unitId String with unit id.
     * 
     * @return void
     * 
     * @access public
     * @author Bruna Chaves Carneiro
     */
    public function deleteNotifications($notificationId, $operatorId, $unitId)
    {
        $sql = "DELETE
                FROM NOTIFICSUPPLY
                WHERE NRNOTIFICSUPPLY = :ID
                        AND CDOPERADOR = :OPERATOR_ID
                        AND CDFILIAL = :UNIT_ID";
                
        $params = array(
            "ID" => $notificationId,
            "OPERATOR_ID" => $operatorId,
            "UNIT_ID" => $unitId
        );
    
        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
    }
}