<?php
namespace Zeedhi\Framework\Annotations;

use Doctrine\Common\Annotations\AnnotationReader;
use Zeedhi\Framework\Annotations\Definitions;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Routing\Parsers\JSONFormat;
use Zeedhi\Framework\Routing\Next\RouteFactory;
use Symfony\Component\Routing\Annotation\Route;

/**
 * Parser class to generate swagger.json from PHP files
 * using doctrine/annotations.
 */
class Parser {
    /**
     * Cleans (nulls, path properties, references, and operations)
     * and fix array indexing (to same properties as above)
     * to finish the associative array into a json parsible.
     *
     * @param array  $json  The json parsed as associative array.
     *
     * @return mixed
     */
    private static function doAfterWork(array &$json) : array {
        foreach ($json as $key => $value) {
            if(is_null($value)){
                unset($json[$key]);
            }
            if(empty($value)){
                unset($json[$key]);
            }
        }
        foreach ($json as $key => $value) {
            if($key === 'paths'){
                foreach ($value as $index => $path) {
                    $value[$path['path']] = $path;
                    unset($value[$index]);
                    unset($value[$path['path']]['path']);
                }
                $json[$key] = $value;
            }
            if($key === 'operations'){
                foreach ($value as $index => $operation) {
                    $type = $operation['type'];
                    $json[$type] = $operation;
                    unset($json[$type]['type']);
                }
                unset($json[$key]);
            }
            if($key === 'ref'){
                $ref = $value['reference'];
                return ['$ref' => $ref];
            }
        }
        foreach ($json as $key => $value) {
            if(is_array($value)){
                $json[$key] = self::doAfterWork($value);
            }
        }
        return $json;
    }

    /**
     * Prepares the openapi.json as a associative array
     * using json_functions, cleans it up, and fix operations,
     * refs, and resources (paths).
     *
     * @param array $api The api as an array of objects.
     *
     * @return array  The prepared json in an associative array state.
     */
    private static function buildJSON(array $api) : array {
        $jsonArray = [];
        $openAPI = new Definitions\OpenAPI;
        $map = ['info', 'openapi', 'components', 'externalDocs', 'paths', 'tags', 'servers'];
        foreach ($map as $property) {
            $openAPI->{$property} = $api[$property];
        }
        $jsonArray = json_decode(json_encode($openAPI), true);
        $jsonArray = self::doAfterWork($jsonArray);
        return $jsonArray;
    }

    /**
     * Returns a new openapi.json.
     *
     * The difference between this method from the buildJSON method
     * is about dealing directly with classes and container.
     *
     * @return string The new openapi.json.
     */
    public static function getJSON() : string {
        $definitions = InstanceManager::getInstance()->getDefinitions();
        $api = [
            'info' =>  null,
            'openapi' => '3.0.0',
            'externalDocs' => null,
            'components' => null,
            'paths' => [],
            'tags' => [],
            'servers' => [],
        ];
        $reader = new AnnotationReader;
        foreach($definitions as $key => $definition){
            $className = $definition->getClass();
            $reflectionClass = new \ReflectionClass($className);
            if($annotations = $reader->getClassAnnotations($reflectionClass)){
                if(($annotation = reset($annotations)) instanceof Definitions\OpenAPI){
                    if($annotation->info){
                        $api['info'] = $annotation->info;
                    }
                    if($annotation->openapi){
                        $api['openapi'] = $annotation->openapi;
                    }
                    if($annotation->components){
                        $api['components'] = $annotation->components;
                    }
                    if($annotation->externalDocs){
                        $api['externalDocs'] = $annotation->externalDocs;
                    }
                    $api['paths'] = array_merge($api['paths'], $annotation->paths ?: []);
                    $api['tags'] = array_merge($api['tags'], $annotation->tags ?: []);
                    $api['servers'] = array_merge($api['servers'], $annotation->servers ?: []);
                }
            }
        }
        $builtJson = [];
        if(!empty($api)){
            $builtJson = self::buildJSON($api);
        }
        return json_encode($builtJson, JSON_PRETTY_PRINT);
    }

    /**
     * Returns the openapi.json as array if it exists or generate a new.
     *
     * @param $metadataSource The metadata dir path.
     * @param $resource       The resource.
     *
     * @return array
     */
    public static function getOldJSON(string $metadataSource, string $resource = null) : array {
        if($filePath = realpath($metadataSource.DIRECTORY_SEPARATOR."openapi.json")){
            $file = file_get_contents($filePath);
            $fileJson = json_decode($file, true);
            if(!$fileJson){
                throw new \Exception(json_last_error_msg());
            }
            if($resource){
                if(!isset($fileJson['paths'])){
                    throw new \Exception("No paths found!");
                }
                foreach($fileJson['paths'] as $key=>$path){
                    if($key === $resource){
                        return $path;
                    }
                }
                throw new \Exception("Couldn't find resource!");
            }
            return $fileJson;
        }
    }

    /**
     * Read the controllers file and register all methods annotated
     * with @RequestMapping on a route.
     *
     * @param string $filePath
     */
    public static function registerControllerRoutes(string $filePath) {
        $instanceManager = InstanceManager::getInstance();
        $nameProvider = $instanceManager->getService('nameProvider');
        $routeFactory = new RouteFactory($nameProvider);
        $reader = new AnnotationReader;
        $controllers = JSONFormat::getRoutesFromFile($filePath);

        foreach($controllers as $controllerId) {
            $controllerInstance = $instanceManager->getService($controllerId);
            $controllerReflection = new \ReflectionClass($controllerInstance);
            $controllerMethods = $controllerReflection->getMethods();
            foreach($controllerMethods as $method) {
                $annotation = $reader->getMethodAnnotations($method, "Zeedhi\Framework\Annotations\RequestMapping");
                if($annotation) {
                    $routeFactory->buildRoute($annotation[0]->uri, $annotation[0]->method, $method->name, $controllerId);
                }
            }
        }
    }
}